#include <iostream>
#include <iomanip>
#include <sstream>
#include <cmath>

using namespace std;

inline long double sqr( long double v ){
	return v * v;
}

class CP {
public:
	int n;
	CP(){}
	CP(int vn): n(vn){}
	~CP(){}

	void set_n(int vn) {
		n = vn;
	}

	long double pfunc(long double x, int k) {
		if (k > 1) {
			return ((2.0 * k - 1.0) * x * pfunc(x, k - 1)
				- (k - 1.0) * pfunc(x, k - 2)) / k;
		}
		else if (k == 1) {
			return x;
		}
		else if (k == 0) {
			return 1.0;
		}else{
			ostringstream sout;
			sout << "Error at pfunc...k = " << k;
			throw sout.str();
		}
	}

	long double operator()(long double x) {
		return pfunc(x, n);
	}
};

class CDP {
public:
	int n;
	CDP() {}
	CDP(int vn) : n(vn) {}
	~CDP() {}

	void set_n(int vn) {
		n = vn;
	}

	long double pfunc(long double x, int k) {
		if (k > 1) {
			return ((2.0 * k - 1.0) * x * pfunc(x, k - 1)
				- (k - 1.0) * pfunc(x, k - 2)) / k;
		}
		else if (k == 1) {
			return x;
		}
		else if (k == 0) {
			return 1.0;
		}
		else {
			ostringstream sout;
			sout << "Error at pfunc...k = " << k;
			throw sout.str();
		}
	}

	long double dpfunc(long double x, int k) {
		if (k > 1) {
			return ((2.0 * k - 1.0)  * (pfunc(x, k - 1) + x * dpfunc(x, k - 1))
				- (k - 1) * dpfunc(x, k - 2)) / k;
		}
		else if (k == 1) {
			return 1.0;
		}
		else if (k == 0) {
			return 0.0;
		}
		else {
			ostringstream sout;
			sout << "Error at dpfunc...k = " << k;
			throw sout.str();
		}
	}

	long double operator()(long double x) {
		return dpfunc(x, n);
	}
};


//     Bisection@ɂ鍪̌vZ    
template<class TF>
void TBisection(TF & f,					 //   ֐
		long double s,				 //   f֐̂Ƃ s > 0.0
		long double c,				 //   ɑ΂֐l
		long double L_b, long double U_b,		 //   ̑݋Ԃ̉Ə
		long double & Root,				 //     f(Root) = c
		long double acc)				 //   x
{
	if ((f(L_b) - c) * (f(U_b) - c) > 0.0) {
		ostringstream sout;
		sout << "Invalid combination of L_b and U_b at TBisection.";
		throw sout.str();
	}
	do {
		long double m = 0.5 * (L_b + U_b);
		long double v = f(m);
		if (s * (v - c) > 0.0) {
			U_b = m;
		}
		else {
			L_b = m;
		}
	} while (abs(U_b - L_b) >= acc);

	Root = 0.5 * (L_b + U_b);
}

//     Newton@ɂ鍪̌vZ   
template<class TF, class TDF>
void TNewton(	long double & x,		//  ̏lA߂l͍ f(x)=c
		 long double c,		//  ֐l
		 TF & f,			//  ֐
		 TDF & df,			//  ֐
		 long double acc)		//	x
{
	int NStep = 0;
	int ck = 0;
	long double x0;
	do {
		x0 = x;
		x = x0 + (c - f(x0)) / df(x0);
	} while (abs(x - x0) >= acc);	
	return;
}

int main() {
	try {
		CP p(9);
		CDP dp(9);
		long double xpos0 = -1.0 + 0.01;
		cout << setprecision(19) << scientific;
		for (int istep = 0; istep < 9; istep++) {
			cout << "istep = " << istep << endl;
			long double ck_v = p(xpos0);
			long double xpos1 = xpos0;
			long double v;
			do {
				xpos1 += 0.01;
				v = p(xpos1);
				if (v * ck_v < 0.0) break;
			} while (true);
			long double xi;
			TBisection<CP>(p, v - ck_v,	0.0, xpos0, xpos1, xi, 1e-7);			
		
			TNewton<CP, CDP>( xi, 0.0, p, dp, 1.0e-19);	
			cout << "xi = " << setw(30) << left << xi;

			long double wi = 2.0 / ((1.0 - sqr(xi)) * sqr(dp(xi)));
			cout << "wi = " << wi << endl;

			xpos0 = xi + 0.01;

		}
	}
	catch (string s){
			cout << s << endl;
	}
	return 0;
}
